"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.minimumOptions = exports.apiEndpointAddress = exports.accessToken = void 0;
const LensPlatformClient_1 = __importDefault(require("./LensPlatformClient"));
const axios_1 = __importDefault(require("axios"));
// A random jwt from https://www.jsonwebtoken.io/
exports.accessToken = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWUsImp0aSI6ImI4Y2RmMmRjLTA3ZmUtNDc5Ny1iOWZkLThmYjlmYTMyZGMyZiIsImlhdCI6MTYxODQ4Mjc3OCwiZXhwIjoxNjE4NDg2Mzc4fQ.h9jJveiwYLPDIX3ZIqB-06QH6CLTDVKToSfWJnwRAgg";
exports.apiEndpointAddress = "http://api.endpoint";
exports.minimumOptions = {
    accessToken: exports.accessToken,
    keyCloakAddress: "",
    keycloakRealm: "",
    apiEndpointAddress: // The realm name, e.g. "lensCloud"
    exports.apiEndpointAddress, // Api endpoint address, e.g. "https://api.k8slens.dev"
};
describe("LensPlatformClient", () => {
    jest.mock("axios");
    // @swc/jest would throw error when compiling this test case.
    it.skip("is a class", () => {
        // @ts-expect-error
        // eslint-disable-next-line new-cap, @typescript-eslint/no-unsafe-return
        expect(() => (0, LensPlatformClient_1.default)()).toThrow("Cannot call a class as a function");
    });
    it("checks options in constructor", () => {
        // @ts-expect-error
        expect(() => new LensPlatformClient_1.default()).toThrow("Options can not be undefined");
        expect(() => new LensPlatformClient_1.default({
            accessToken: undefined,
            getAccessToken: undefined,
            keyCloakAddress: "",
            keycloakRealm: "",
            apiEndpointAddress: "",
        })).toThrow("Both accessToken undefined or getAccessToken are undefined");
    });
    it("can `new LensPlatformClient` with minimum (but valid) options", () => {
        expect(() => new LensPlatformClient_1.default(exports.minimumOptions)).not.toThrow();
    });
    it(".authHeader", () => {
        const lensPlatformClient = new LensPlatformClient_1.default(exports.minimumOptions);
        expect(lensPlatformClient.authHeader).toEqual({
            Authorization: `Bearer ${exports.accessToken}`,
        });
    });
    describe("proxied version of fetch", () => {
        it("adds Authorization header", async () => {
            const expectedHeaders = { Authorization: `Bearer ${exports.accessToken}` };
            const lensPlatformClient = new LensPlatformClient_1.default({
                ...exports.minimumOptions,
            });
            const spies = [
                jest.spyOn(axios_1.default, "get"),
                jest.spyOn(axios_1.default, "head"),
                jest.spyOn(axios_1.default, "delete"),
            ];
            const _fetch = lensPlatformClient.fetch;
            try {
                await Promise.all([
                    _fetch.get(exports.apiEndpointAddress),
                    _fetch.head(exports.apiEndpointAddress),
                    _fetch.delete(exports.apiEndpointAddress),
                ]);
            }
            catch {
                // Do not handle exceptions
            }
            finally {
                spies.forEach((spy) => {
                    expect(spy).toBeCalledWith(exports.apiEndpointAddress, { headers: expectedHeaders });
                    spy.mockRestore();
                });
            }
        });
        describe("when unauthenticated", () => {
            it("doesn't add Authorization header", async () => {
                const expectedHeaders = {};
                const lensPlatformClient = new LensPlatformClient_1.default({
                    ...exports.minimumOptions,
                });
                const spies = [
                    jest.spyOn(axios_1.default, "get"),
                    jest.spyOn(axios_1.default, "head"),
                    jest.spyOn(axios_1.default, "delete"),
                ];
                const _fetch = lensPlatformClient.fetch;
                try {
                    await Promise.all([
                        _fetch.get(exports.apiEndpointAddress, {
                            unauthenticated: true,
                        }),
                        _fetch.head(exports.apiEndpointAddress, {
                            unauthenticated: true,
                        }),
                        _fetch.delete(exports.apiEndpointAddress, {
                            unauthenticated: true,
                        }),
                    ]);
                }
                catch {
                    // Do not handle exceptions
                }
                finally {
                    spies.forEach((spy) => {
                        expect(spy).toBeCalledWith(exports.apiEndpointAddress, {
                            headers: expectedHeaders,
                            unauthenticated: true,
                        });
                        spy.mockRestore();
                    });
                }
            });
        });
        it("doesn't add Authorization header if no token", async () => {
            const lensPlatformClient = new LensPlatformClient_1.default({
                ...exports.minimumOptions,
                accessToken: "",
                getAccessToken: async () => Promise.resolve(""),
            });
            const spy = jest.spyOn(axios_1.default, "get");
            const _fetch = lensPlatformClient.fetch;
            try {
                await _fetch.get(exports.apiEndpointAddress);
            }
            catch {
                // Do not handle exceptions
            }
            finally {
                expect(spy).toBeCalledWith(exports.apiEndpointAddress, { headers: {} });
                spy.mockRestore();
            }
        });
        it("adds Authorization header with body", async () => {
            const expectedHeaders = { Authorization: `Bearer ${exports.accessToken}` };
            const lensPlatformClient = new LensPlatformClient_1.default({
                ...exports.minimumOptions,
            });
            const spies = [
                jest.spyOn(axios_1.default, "post"),
                jest.spyOn(axios_1.default, "put"),
                jest.spyOn(axios_1.default, "patch"),
            ];
            const _fetch = lensPlatformClient.fetch;
            try {
                await Promise.all([
                    _fetch.post(exports.apiEndpointAddress),
                    _fetch.put(exports.apiEndpointAddress),
                    _fetch.patch(exports.apiEndpointAddress),
                ]);
            }
            catch {
                // Do not handle exceptions
            }
            finally {
                spies.forEach((spy) => {
                    expect(spy).toBeCalledWith(exports.apiEndpointAddress, undefined, { headers: expectedHeaders });
                    spy.mockRestore();
                });
            }
        });
        it("merged request options", async () => {
            const extraOptions = { withCredentials: true };
            const expectedHeaders = { Authorization: `Bearer ${exports.accessToken}` };
            const lensPlatformClient = new LensPlatformClient_1.default({
                ...exports.minimumOptions,
            });
            const spies = [
                jest.spyOn(axios_1.default, "get"),
                jest.spyOn(axios_1.default, "head"),
                jest.spyOn(axios_1.default, "delete"),
            ];
            const _fetch = lensPlatformClient.fetch;
            try {
                await Promise.all([
                    _fetch.get(exports.apiEndpointAddress, extraOptions),
                    _fetch.head(exports.apiEndpointAddress, extraOptions),
                    _fetch.delete(exports.apiEndpointAddress, extraOptions),
                ]);
            }
            catch {
                // Do not handle exceptions
            }
            finally {
                spies.forEach((spy) => {
                    expect(spy).toBeCalledWith(exports.apiEndpointAddress, {
                        headers: expectedHeaders,
                        ...extraOptions,
                    });
                    spy.mockRestore();
                });
            }
        });
        it("merged request options with body", async () => {
            const extraOptions = { withCredentials: true };
            const expectedHeaders = { Authorization: `Bearer ${exports.accessToken}` };
            const lensPlatformClient = new LensPlatformClient_1.default({
                ...exports.minimumOptions,
            });
            const spies = [
                jest.spyOn(axios_1.default, "post"),
                jest.spyOn(axios_1.default, "put"),
                jest.spyOn(axios_1.default, "patch"),
            ];
            const _fetch = lensPlatformClient.fetch;
            try {
                await Promise.all([
                    _fetch.post(exports.apiEndpointAddress, undefined, extraOptions),
                    _fetch.put(exports.apiEndpointAddress, undefined, extraOptions),
                    _fetch.patch(exports.apiEndpointAddress, undefined, extraOptions),
                ]);
            }
            catch {
                // Do not handle exceptions
            }
            finally {
                spies.forEach((spy) => {
                    expect(spy).toBeCalledWith(exports.apiEndpointAddress, undefined, {
                        headers: expectedHeaders,
                        ...extraOptions,
                    });
                    spy.mockRestore();
                });
            }
        });
        it("merged headers", async () => {
            const defaultHeaders = { "X-Consumer-Id": "xx-yy-zz" };
            const extraHeader = { "X-An-Example": "Header" };
            const expectedHeaders = {
                Authorization: `Bearer ${exports.accessToken}`,
                ...extraHeader,
                ...defaultHeaders,
            };
            const lensPlatformClient = new LensPlatformClient_1.default({
                ...exports.minimumOptions,
                ...{
                    defaultHeaders: { "X-Consumer-Id": "xx-yy-zz" },
                },
            });
            const spies = [
                jest.spyOn(axios_1.default, "get"),
                jest.spyOn(axios_1.default, "head"),
                jest.spyOn(axios_1.default, "delete"),
            ];
            const _fetch = lensPlatformClient.fetch;
            try {
                await Promise.all([
                    _fetch.get(exports.apiEndpointAddress, { headers: extraHeader }),
                    _fetch.head(exports.apiEndpointAddress, { headers: extraHeader }),
                    _fetch.delete(exports.apiEndpointAddress, { headers: extraHeader }),
                ]);
            }
            catch {
                // Do not handle exceptions
            }
            finally {
                spies.forEach((spy) => {
                    expect(spy).toBeCalledWith(exports.apiEndpointAddress, { headers: expectedHeaders });
                    spy.mockRestore();
                });
            }
        });
        it("merged headers with body", async () => {
            const defaultHeaders = { "X-Consumer-Id": "xx-yy-zz" };
            const extraHeader = { "X-An-Example": "Header" };
            const expectedHeaders = {
                Authorization: `Bearer ${exports.accessToken}`,
                ...extraHeader,
                ...defaultHeaders,
            };
            const lensPlatformClient = new LensPlatformClient_1.default({
                ...exports.minimumOptions,
                ...{
                    defaultHeaders: { "X-Consumer-Id": "xx-yy-zz" },
                },
            });
            const spies = [
                jest.spyOn(axios_1.default, "post"),
                jest.spyOn(axios_1.default, "put"),
                jest.spyOn(axios_1.default, "patch"),
            ];
            const _fetch = lensPlatformClient.fetch;
            try {
                await Promise.all([
                    _fetch.post(exports.apiEndpointAddress, {}, { headers: extraHeader }),
                    _fetch.put(exports.apiEndpointAddress, {}, { headers: extraHeader }),
                    _fetch.patch(exports.apiEndpointAddress, {}, { headers: extraHeader }),
                ]);
            }
            catch {
                // Do not handle exceptions
            }
            finally {
                spies.forEach((spy) => {
                    expect(spy).toBeCalledWith(exports.apiEndpointAddress, {}, { headers: expectedHeaders });
                    spy.mockRestore();
                });
            }
        });
        it("merged headers and request options at the same time", async () => {
            const extraHeader = { "X-An-Example": "Header" };
            const extraOption = { json: { an: "example_extra_option" } };
            const lensPlatformClient = new LensPlatformClient_1.default({
                ...exports.minimumOptions,
            });
            const spies = [
                jest.spyOn(axios_1.default, "get"),
                jest.spyOn(axios_1.default, "head"),
                jest.spyOn(axios_1.default, "delete"),
            ];
            const _fetch = lensPlatformClient.fetch;
            try {
                await Promise.all([
                    _fetch.get(exports.apiEndpointAddress, { ...extraOption, headers: extraHeader }),
                    _fetch.head(exports.apiEndpointAddress, { ...extraOption, headers: extraHeader }),
                    _fetch.delete(exports.apiEndpointAddress, { ...extraOption, headers: extraHeader }),
                ]);
            }
            catch {
                // Do not handle exceptions
            }
            finally {
                spies.forEach((spy) => {
                    expect(spy).toBeCalledWith(exports.apiEndpointAddress, {
                        headers: {
                            ...extraHeader,
                            ...{ Authorization: `Bearer ${exports.accessToken}` },
                        },
                        ...extraOption,
                    });
                    spy.mockRestore();
                });
            }
        });
        it("merged headers and request options at the same time with body", async () => {
            const extraHeader = { "X-An-Example": "Header" };
            const extraOption = { json: { an: "example_extra_option" } };
            const lensPlatformClient = new LensPlatformClient_1.default({
                ...exports.minimumOptions,
            });
            const spies = [
                jest.spyOn(axios_1.default, "post"),
                jest.spyOn(axios_1.default, "put"),
                jest.spyOn(axios_1.default, "patch"),
            ];
            const _fetch = lensPlatformClient.fetch;
            try {
                await Promise.all([
                    _fetch.post(exports.apiEndpointAddress, { a: 1 }, { ...extraOption, headers: extraHeader }),
                    _fetch.put(exports.apiEndpointAddress, { a: 1 }, { ...extraOption, headers: extraHeader }),
                    _fetch.patch(exports.apiEndpointAddress, { a: 1 }, { ...extraOption, headers: extraHeader }),
                ]);
            }
            catch {
                // Do not handle exceptions
            }
            finally {
                spies.forEach((spy) => {
                    expect(spy).toBeCalledWith(exports.apiEndpointAddress, { a: 1 }, {
                        headers: {
                            ...extraHeader,
                            ...{ Authorization: `Bearer ${exports.accessToken}` },
                        },
                        ...extraOption,
                    });
                    spy.mockRestore();
                });
            }
        });
    });
});
