"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require(".");
const LensPlatformClient_test_1 = require("./LensPlatformClient.test");
const Permissions_1 = require("./Permissions");
// Current user (Owner)
const ownerUser = {
    id: "1234567890",
};
// Admin user
const adminUser = {
    id: "1",
};
// Member user
const memberUser = {
    id: "2",
};
// Random user
const mockUser4 = {
    id: "3",
};
// Member of Space, but not team
const mockUser5 = {
    id: "4",
};
const ownerTeam = {
    kind: "Owner",
    id: "mt1",
    name: "mt1",
    description: "mt1",
    users: [ownerUser],
    spaceId: "ms1",
};
const adminTeam = {
    kind: "Admin",
    id: "mt2",
    name: "mt2",
    description: "mt2",
    users: [adminUser],
    spaceId: "ms1",
};
const normalTeam = {
    kind: "Normal",
    id: "mt3",
    name: "mt3",
    description: "mt3",
    users: [memberUser],
    spaceId: "ms1",
};
const mockSpace1 = {
    id: "ms1",
    name: "ms1",
    description: "ms1",
    kind: "Team",
    users: [ownerUser, adminUser, memberUser, mockUser5],
    teams: [ownerTeam, adminTeam, normalTeam],
    createdById: ownerUser.id,
};
const mockK8sCluster1 = {
    id: "mk1",
    createdById: memberUser.id,
    name: "",
    kind: "K8sCluster",
};
const mockDevCluster1 = {
    id: "mk1",
    createdById: memberUser.id,
    name: "",
    kind: "K8sCluster",
    metadata: {
        labels: {
            devCluster: "true",
        },
    },
};
const mockDevCluster2 = {
    id: "mk1",
    createdById: mockUser5.id,
    name: "",
    kind: "K8sCluster",
    metadata: {
        labels: {
            devCluster: "true",
        },
    },
};
const invitationToBeRevokedByMockUser3 = "invitation_id_to_be_revoked_by_user_3";
const invitationIdsCreatedByMockUser3 = [invitationToBeRevokedByMockUser3, "another_invitation_id"];
describe("PermissionsService", () => {
    let client;
    beforeAll(() => {
        client = new _1.LensPlatformClient(LensPlatformClient_test_1.minimumOptions);
    });
    it("should contain a reference to `PermissionsService`", () => {
        expect(client.permission).toBeDefined();
    });
    describe(".getRole", () => {
        it("recognizes Owner, Admin and Member roles", () => {
            expect(client.permission.getRole(mockSpace1, ownerUser.id)).toEqual(_1.Roles.Owner);
            expect(client.permission.getRole(mockSpace1, adminUser.id)).toEqual(_1.Roles.Admin);
            expect(client.permission.getRole(mockSpace1, memberUser.id)).toEqual(_1.Roles.Member);
            // MockUser5 is Space member but is not in any team
            expect(client.permission.getRole(mockSpace1, mockUser5.id)).toEqual(_1.Roles.Member);
        });
        it("recognizes lack of role", () => {
            expect(client.permission.getRole(mockSpace1, mockUser4.id)).toEqual(_1.Roles.None);
        });
    });
    describe(".canSpace", () => {
        it("recognizes owner privileges", () => {
            expect(client.permission.canSpace(_1.Actions.DeleteSpace, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.RenameSpace, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.CreateTeam, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.DeleteTeam, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.PatchTeam, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.PatchInvitation, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.RevokeInvitation, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.CreateInvitation, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.PatchSpace, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.AddInvitationDomain, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.DeleteInvitationDomain, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.ChangeSpacePlan, mockSpace1, ownerUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.GetBillingPageToken, mockSpace1, ownerUser.id)).toBeTruthy();
        });
        it("can't delete Personal Space", () => {
            expect(client.permission.canSpace(_1.Actions.DeleteSpace, {
                ...mockSpace1,
                kind: "Personal",
            }, ownerUser.id)).toBeFalsy();
        });
        it("can't rename Personal Space", () => {
            expect(client.permission.canSpace(_1.Actions.RenameSpace, {
                ...mockSpace1,
                kind: "Personal",
            }, ownerUser.id)).toBeFalsy();
        });
        it("recognizes admin privileges", () => {
            expect(client.permission.canSpace(_1.Actions.DeleteSpace, mockSpace1, adminUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.RenameSpace, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.CreateTeam, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.DeleteTeam, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.PatchTeam, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.PatchInvitation, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.RevokeInvitation, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.CreateInvitation, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.PatchSpace, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.AddInvitationDomain, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.DeleteInvitationDomain, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.GetBillingPageToken, mockSpace1, adminUser.id)).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.ChangeSpacePlan, mockSpace1, adminUser.id)).toBeFalsy();
        });
        it("recognizes member privileges", () => {
            expect(client.permission.canSpace(_1.Actions.CreateInvitation, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.DeleteSpace, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.CreateTeam, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.DeleteTeam, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.PatchTeam, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.RenameSpace, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.PatchInvitation, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.RevokeInvitation, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.RevokeInvitation, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.RevokeInvitation, mockSpace1, memberUser.id, {
                invitationId: invitationToBeRevokedByMockUser3,
                invitationIdsCreatedByUserId: invitationIdsCreatedByMockUser3,
            })).toBeTruthy();
            expect(client.permission.canSpace(_1.Actions.PatchSpace, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.AddInvitationDomain, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.DeleteInvitationDomain, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.GetBillingPageToken, mockSpace1, memberUser.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.ChangeSpacePlan, mockSpace1, memberUser.id)).toBeFalsy();
        });
        it("recognizes lack of privileges (random unrelated user test)", () => {
            expect(client.permission.canSpace(_1.Actions.DeleteSpace, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.CreateTeam, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.DeleteTeam, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.PatchTeam, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.PatchInvitation, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.CreateInvitation, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.PatchSpace, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.RenameSpace, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.AddInvitationDomain, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.DeleteInvitationDomain, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.GetBillingPageToken, mockSpace1, mockUser4.id)).toBeFalsy();
            expect(client.permission.canSpace(_1.Actions.ChangeSpacePlan, mockSpace1, mockUser4.id)).toBeFalsy();
        });
    });
    describe(".canTeam", () => {
        describe("AddUser", () => {
            it("can add user to team as Owner", () => {
                var _a;
                expect(client.permission.canTeam(Permissions_1.TeamActions.AddUser, mockSpace1, ownerTeam, (_a = ownerUser.id) !== null && _a !== void 0 ? _a : "")).toBeTruthy();
            });
            it("can add user to Owner team of Personal Space", () => {
                var _a;
                expect(client.permission.canTeam(Permissions_1.TeamActions.AddUser, {
                    ...mockSpace1,
                    kind: "Personal",
                }, ownerTeam, (_a = adminUser.id) !== null && _a !== void 0 ? _a : "")).toBeFalsy();
            });
            it("admin user can't add user to owner team", () => {
                var _a;
                expect(client.permission.canTeam(Permissions_1.TeamActions.AddUser, mockSpace1, ownerTeam, (_a = adminUser.id) !== null && _a !== void 0 ? _a : "")).toBeFalsy();
            });
            it("admin user can add user to admin team", () => {
                var _a;
                expect(client.permission.canTeam(Permissions_1.TeamActions.AddUser, mockSpace1, adminTeam, (_a = adminUser.id) !== null && _a !== void 0 ? _a : "")).toBeTruthy();
            });
            it("admin user can add user to normal team", () => {
                var _a;
                expect(client.permission.canTeam(Permissions_1.TeamActions.AddUser, mockSpace1, normalTeam, (_a = adminUser.id) !== null && _a !== void 0 ? _a : "")).toBeTruthy();
            });
            it("member user can't add user to owner team", () => {
                var _a;
                expect(client.permission.canTeam(Permissions_1.TeamActions.AddUser, mockSpace1, ownerTeam, (_a = memberUser.id) !== null && _a !== void 0 ? _a : "")).toBeFalsy();
            });
            it("member user can't add user to admin team", () => {
                var _a;
                expect(client.permission.canTeam(Permissions_1.TeamActions.AddUser, mockSpace1, adminTeam, (_a = memberUser.id) !== null && _a !== void 0 ? _a : "")).toBeFalsy();
            });
            it("member user can't add user to normal team", () => {
                var _a, _b;
                expect(client.permission.canTeam(Permissions_1.TeamActions.AddUser, mockSpace1, normalTeam, (_a = memberUser.id) !== null && _a !== void 0 ? _a : "", (_b = memberUser.id) !== null && _b !== void 0 ? _b : "")).toBeFalsy();
            });
        });
        describe("RemoveUser", () => {
            it("can remove user from team as Owner", () => {
                var _a, _b;
                expect(client.permission.canTeam(Permissions_1.TeamActions.RemoveUser, mockSpace1, ownerTeam, (_a = ownerUser.id) !== null && _a !== void 0 ? _a : "", (_b = ownerUser.id) !== null && _b !== void 0 ? _b : "")).toBeTruthy();
            });
            it("can't remove itself from Owner team of Personal Space", () => {
                var _a, _b;
                expect(client.permission.canTeam(Permissions_1.TeamActions.RemoveUser, {
                    ...mockSpace1,
                    kind: "Personal",
                }, ownerTeam, (_a = ownerUser.id) !== null && _a !== void 0 ? _a : "", (_b = ownerUser.id) !== null && _b !== void 0 ? _b : "")).toBeFalsy();
            });
            it("admin team can't remove user from owner team", () => {
                var _a, _b;
                expect(client.permission.canTeam(Permissions_1.TeamActions.RemoveUser, mockSpace1, ownerTeam, (_a = adminUser.id) !== null && _a !== void 0 ? _a : "", (_b = ownerUser.id) !== null && _b !== void 0 ? _b : "")).toBeFalsy();
            });
            it("admin team can remove user from admin team", () => {
                var _a, _b;
                expect(client.permission.canTeam(Permissions_1.TeamActions.RemoveUser, mockSpace1, adminTeam, (_a = adminUser.id) !== null && _a !== void 0 ? _a : "", (_b = adminUser.id) !== null && _b !== void 0 ? _b : "")).toBeTruthy();
            });
            it("admin team can remove user from normal team", () => {
                var _a, _b;
                expect(client.permission.canTeam(Permissions_1.TeamActions.RemoveUser, mockSpace1, normalTeam, (_a = adminUser.id) !== null && _a !== void 0 ? _a : "", (_b = memberUser.id) !== null && _b !== void 0 ? _b : "")).toBeTruthy();
            });
            it("member user can't remove user to owner team", () => {
                var _a, _b;
                expect(client.permission.canTeam(Permissions_1.TeamActions.RemoveUser, mockSpace1, ownerTeam, (_a = memberUser.id) !== null && _a !== void 0 ? _a : "", (_b = ownerUser.id) !== null && _b !== void 0 ? _b : "")).toBeFalsy();
            });
            it("member user can't remove user to admin team", () => {
                var _a, _b;
                expect(client.permission.canTeam(Permissions_1.TeamActions.RemoveUser, mockSpace1, adminTeam, (_a = memberUser.id) !== null && _a !== void 0 ? _a : "", (_b = adminUser.id) !== null && _b !== void 0 ? _b : "")).toBeFalsy();
            });
            it("member user can't remove user to normal team", () => {
                var _a, _b;
                expect(client.permission.canTeam(Permissions_1.TeamActions.RemoveUser, mockSpace1, normalTeam, (_a = memberUser.id) !== null && _a !== void 0 ? _a : "", (_b = memberUser.id) !== null && _b !== void 0 ? _b : "")).toBeFalsy();
            });
        });
    });
    describe(".canK8sCluster", () => {
        describe("DeleteK8sCluster", () => {
            it("user can delete cluster created by the user", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.DeleteK8sCluster, mockSpace1, mockK8sCluster1, memberUser.id)).toBeTruthy();
            });
            it("Admin can delete", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.DeleteK8sCluster, mockSpace1, mockK8sCluster1, adminUser.id)).toBeTruthy();
            });
            it("Owner can delete", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.DeleteK8sCluster, mockSpace1, mockK8sCluster1, ownerUser.id)).toBeTruthy();
            });
            it("non-owner user can't delete", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.DeleteK8sCluster, mockSpace1, mockK8sCluster1, mockUser4.id)).toBeFalsy();
            });
        });
        describe("AccessK8sCluster", () => {
            it("user can access non-devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.AccessK8sCluster, mockSpace1, mockK8sCluster1, memberUser.id)).toBeTruthy();
            });
            it("user can access own devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.AccessK8sCluster, mockSpace1, mockDevCluster1, memberUser.id)).toBeTruthy();
            });
            it("user can access own devCluster 2", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.AccessK8sCluster, mockSpace1, mockDevCluster2, mockUser5.id)).toBeTruthy();
            });
            it("member can't access another user's devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.AccessK8sCluster, mockSpace1, mockDevCluster1, mockUser5.id)).toBeFalsy();
            });
            it("member can't access another user's devCluster 2", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.AccessK8sCluster, mockSpace1, mockDevCluster2, memberUser.id)).toBeFalsy();
            });
            it("admin can access own devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.AccessK8sCluster, mockSpace1, mockDevCluster1, adminUser.id)).toBeTruthy();
            });
            it("admin can access another user's devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.AccessK8sCluster, mockSpace1, mockDevCluster2, adminUser.id)).toBeTruthy();
            });
            it("owner can access own devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.AccessK8sCluster, mockSpace1, mockDevCluster1, ownerUser.id)).toBeTruthy();
            });
            it("owner can access another user's devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.AccessK8sCluster, mockSpace1, mockDevCluster2, ownerUser.id)).toBeTruthy();
            });
        });
        describe("PatchK8sCluster", () => {
            it("user can patch non-devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.PatchK8sCluster, mockSpace1, mockK8sCluster1, memberUser.id)).toBeTruthy();
            });
            it("user can patch own devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.PatchK8sCluster, mockSpace1, mockDevCluster1, memberUser.id)).toBeTruthy();
            });
            it("user can patch own devCluster 2", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.PatchK8sCluster, mockSpace1, mockDevCluster2, mockUser5.id)).toBeTruthy();
            });
            it("member can't patch another user's devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.PatchK8sCluster, mockSpace1, mockDevCluster1, mockUser5.id)).toBeFalsy();
            });
            it("member can't patch another user's devCluster 2", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.PatchK8sCluster, mockSpace1, mockDevCluster2, memberUser.id)).toBeFalsy();
            });
            it("admin can patch own devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.PatchK8sCluster, mockSpace1, mockDevCluster1, adminUser.id)).toBeTruthy();
            });
            it("admin can patch another user's devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.PatchK8sCluster, mockSpace1, mockDevCluster2, adminUser.id)).toBeTruthy();
            });
            it("owner can patch own devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.PatchK8sCluster, mockSpace1, mockDevCluster1, ownerUser.id)).toBeTruthy();
            });
            it("owner can patch another user's devCluster", () => {
                expect(client.permission.canK8sCluster(_1.K8sClusterActions.PatchK8sCluster, mockSpace1, mockDevCluster2, ownerUser.id)).toBeTruthy();
            });
        });
    });
});
